#!perl

package Sophos::Reports::DataManip;

use strict;
use warnings;
use Data::Pivot ();
use base qw(Exporter);
use Sophos::Reports::Util qw(duration_tick_boundary);
use Clone qw(clone);

our @EXPORT_OK = qw(rotate min_max pie_chart_sum sum rotate_pivot_on_1 aggregate_to_boundary setup_aggregation dashboard_sum aggregate_and_average);

return 1;

# interlace two data sets
# note: this interlaces based on the value of $data->[n]->[0].
sub interlace {
    my($data, $headers) = @_;
    
    my(@set1, @set2, @rv);
    @set2 = (@$data);
    @set1 = splice(@set2, 0, scalar(@set2 / 2));

    while(@set1 || @set2) {
        if(@set1) {
            push(@rv, shift(@set1));
            while(@set1 && $rv[$#rv][0] eq $set1[0][0]) {
                push(@rv, shift(@set1));
            }
        }

        if(@set2) {
            push(@rv, shift(@set2));
            while(@set2 && $rv[$#rv][0] eq $set2[0][0]) {
                push(@rv, shift(@set2));
            }
        }
    }
    
    return \@rv;
}

sub no_manip {
    my($data, $headers) = @_;
    return $data;
}

sub pivot_on_1 {
    my($data, $headers) = @_;
     
    my(@rv) = Data::Pivot::pivot(
        table           => $data,
        headings        => @$headers[0],
        pivot_column    => 1,
    );
    
    return \@rv;   
}

sub percentage {
    my ($data, $headers) = @_;

    for my $i (0 .. @{$data->[0]} - 1) {
	my $total = 0;
	$total += $_->[$i] foreach @$data;
	$total ||= 1;
	$_->[$i] = 100 * int($_->[$i]/$total) foreach @$data;
    }
    
    return $data;
}

sub rotate {
    my($data, $headers) = @_;
    
    $headers = @$headers[0];
    
    my(@rv);
    foreach my $i ($[ .. $#$headers) {
        push(@rv, []);
    }
    
    foreach my $i (@$data) {
        foreach my $j ($[ .. $#$headers) {
            push(@{$rv[$j]}, $i->[$j]);
        }
    }
    
    return \@rv;
}

sub rotate_pivot_on_1 {
    my($data, $headers) = @_;
    $data = pivot_on_1($data, $headers);
    $data = rotate($data, $headers);
    return $data;
}

sub interlace_rotate_pivot_on_1 {
    my($data, $headers) = @_;
    $data = interlace($data, $headers);
    $data = rotate_pivot_on_1($data, $headers);
    return $data;
}

sub sum {
    my $row = shift;
    my $a = 0;
    $a += $_ foreach @$row;
    return $a;
}

sub pie_chart_sum {
    my($data, $headers) = @_;
    my @sum;
    
    for(my $i = 1; $i <= $#$data; $i++) {
        my $row = $data->[$i];
        push(@sum, sum($row));
    }
    
    my(@newh) = (@$headers[0]);
    @newh = @{$newh[0]};
    
    my @rv = (\@newh, \@sum);
    
    return \@rv;
}

sub dashboard_sum {
    my($data) = @_;
    my $rv = [ $data->[0], [] ];
    foreach my $x ($[ .. $#{$data->[0]}) {
        my $sum = 0;
        $sum += ($data->[$_]->[$x] || 0) foreach (1 .. $#$data);
        push(@{$rv->[1]}, $sum);
    }
    return $rv;
}

sub aggregate_and_average_per_second{
	my($data, $headers, $boundary) = @_;
	return aggregate_and_average($data, $headers, $boundary, 300);
}

# this method is used to calculate the performance 
# reports as we need to get an average per time period
# which contains data
sub aggregate_and_average {
    my($data, $headers, $boundary, $time_period) = @_;
    $data = clone($data);
    if(!$time_period){
	$time_period = 1;
    }
    my($columns, $rows, $start, $end) = setup_aggregation($data);
    my(@rv, @accum);
    # store the number of variables we've retrieved from the database
    # add a value to allow us to ignore the first row
    my $first_row = 1;
    # initialize empty arrays to contain the data
    push(@rv, []) foreach (0 .. $rows - 1);
    push(@accum, 0) foreach (0 .. $rows - 1);
    my $val_counter = 0;

    $accum[0] = $data->[0]->[0];
    # check to see whether or not we've incremented the counter for a
    # given column
    my $val_counter_incremented = 0;
    
    while($columns) {
        shift(@{$data->[0]});
        $val_counter_incremented = 0;
        foreach my $i (($[+1) .. $rows - 1) {
       	    my $val = shift(@{$data->[$i]});   
	    # if a value exists, add it to the cumulative total
	    # and increment the counter
            if($val){
                $accum[$i] += $val;
                if(!$val_counter_incremented){
    		    $val_counter ++;
                    # don't increment the counter again for this column
                    $val_counter_incremented = 1;
                }
            }
        }
        if($data->[0]->[0]) {
            if(($data->[0]->[0] - $accum[0]) >= $boundary) {
                $first_row = 1;
		foreach my $i ($[ .. $rows - 1) {
			if($val_counter && !$first_row){
                   		push(@{$rv[$i]}, sprintf("%.2f", ($accum[$i] / ($val_counter * $time_period)))); 
		                # reset the counter for number of values for this time period
		    	}else{
				$first_row = 0;
				push(@{$rv[$i]}, ($accum[$i]));
		    	}
                    	$accum[$i] = 0;
                }
		$val_counter = 0;
                $accum[0] = $data->[0]->[0];
            }
        }
        $columns--;
    }
    $first_row = 1;
    foreach my $i ($[ .. $rows - 1) {
        if($val_counter && !$first_row){
                push(@{$rv[$i]}, (sprintf("%.2f", ($accum[$i] / $val_counter))));
        }else{
                $first_row = 0;
        	push(@{$rv[$i]}, ($accum[$i]));
        }
        $accum[$i] = 0;
    }
    return \@rv;
}

sub aggregate_and_sort {
    my($data, $headers, $boundary) = @_;
    $data = aggregate_to_boundary($data, $headers, $boundary);
    my ($array_total, $date_row, $index, 
        @sort_order, @sortables, 
        $array_sorted, $tmp_value);
    foreach my $subarray (@$data) {
        $array_total = 0;
        foreach my $value (@$subarray) {
            if($value) {
                $array_total += $value;
            }
        }
        push(@sortables, $array_total);
    }
    
    push(@sort_order, $_) foreach (0 .. @sortables);
    $array_sorted = 0;
    # we do our own sort to get the initial order and pass it back 
    # to wherever we display our data
    while ((@sortables > 1) && (!$array_sorted)) {
        $array_sorted = 1;
        for (my $i=1; $i < @sortables; $i++) {
            if (($i != (@sortables - 1)) && ($sortables[$i] < $sortables[$i + 1])) {
                $tmp_value = $sortables[$i];
                $sortables[$i] = $sortables[$i + 1];
                $sortables[$i + 1] = $tmp_value;          
                $tmp_value = $sort_order[$i];
                $sort_order[$i] = $sort_order[$i + 1];
                $sort_order[$i + 1] = $tmp_value;          
                $array_sorted = 0;
            } elsif (($i == (@sortables - 1)) && ($sortables[$i] > $sortables[$i - 1])) {
                $tmp_value = $sortables[$i];
                $sortables[$i] = $sortables[$i - 1];
                $sortables[$i - 1] = $tmp_value;          
                $tmp_value = $sort_order[$i];
                $sort_order[$i] = $sort_order[$i - 1];
                $sort_order[$i - 1] = $tmp_value;          
                $array_sorted = 0;
            }
        }
    }
    return ($data, @sort_order);
}

sub aggregate_percent_to_boundary {
    my $data = aggregate_to_boundary(@_);

    for my $i (0 .. $#{@{$data->[0]}}) {
	my $total = 0;
	$total += $data->[$_][$i] foreach 1 .. $#{@$data};
	$total ||= 1;

	my $remainder = 100;
	for my $j (1 .. $#{@$data} - 1) {
	    $remainder -= $data->[$j][$i] = int(100 * $data->[$j][$i]/$total);
	}
	$data->[-1][$i] = $remainder;
    }

    return $data;
}

sub aggregate_to_boundary {
    my($data, $headers, $boundary) = @_;
    $data = clone($data);
    my($columns, $rows, $start, $end) = setup_aggregation($data);
    my(@rv, @accum);
    push(@rv, []) foreach (1 .. $rows);
    push(@accum, 0) foreach (1 .. $rows);

    $accum[0] = $data->[0]->[0];
    while ($columns) {
        shift(@{$data->[0]});
        foreach my $i (($[+1) .. $rows - 1) {
            my $val = shift(@{$data->[$i]});
	    if($val){
	            $accum[$i] += $val;
	    }
        }
	# added check for existence of the value
        if($data->[0]->[0]) {
            if(($data->[0]->[0] - $accum[0]) >= $boundary) {
                foreach my $i ($[ .. $rows - 1) {
                    push(@{$rv[$i]}, $accum[$i]);
                    $accum[$i] = 0;
                }
                $accum[0] = $data->[0]->[0];
            }
	}
        $columns--;
    }
    foreach my $i ($[ .. $rows - 1) {
        push(@{$rv[$i]}, $accum[$i]);
       $accum[$i] = 0;
    }
    return \@rv;
}

sub setup_aggregation {
    my ($data, $start, $end) = @_;
    my $columns = scalar(@{$data->[0]});
    my $rows = scalar(@$data);
    if(!defined($start)) {
        $start = $data->[0]->[0];
    }
    if(!defined($end)) {
        $end = $data->[0]->[$columns-1];
    }
    
    return ($columns, $rows, $start, $end);
}

sub sum_row_elements {
    my $row = shift;
    my $total = 0;
    my $c;
    foreach $c (@$row) {
        $total += $c;
    }
    return $total;
}

sub min_max {
    my $data = shift;
    my $min = 1E10;
    my $max = -1E10;
    my ($columns, $rows) = setup_aggregation($data);
    my @accum;
    push(@accum, 0) foreach (1 .. $#{@$data[0]});
    for(my $row=1; $row < $rows; $row ++) {
        for(my $col=0; $col < $columns; $col ++) {
            $accum[$col] += $data->[$row]->[$col];
        }
    }
    @accum = sort @accum;
    $max = $accum[$#{@$data[0]}];
    $min = $accum[0];
    return($min, $max);
}
