#!perl

package Sophos::Reports::Graphic::Dashboard;

use strict;
use warnings;
use base q(GD::Graph::mixed);
use Math::Gradient qw(multi_array_gradient);
use Sophos::GD::Gradient qw(apply_gradient);

our %Defaults = (
    gradient                    =>      [
        [ 0, 0, 0, 0 ], [ 0, 0, 0, 32 ], [ 0, 0, 0, 127 ]
    ],
    gradient_angle              =>      0,
    gradient_steps              =>      200,
    gradient_opacity            =>      0.3,
);

return 1;

sub initialise {
    my $self = shift;
    while(my($k, $v) = each(%Defaults)) {
        $self->set($k,$v) unless defined($self->{$k});
    }
    return $self->SUPER::initialise(@_);
}

sub _has_default {
    my($self, $attr) = @_;
    return 1 if exists $Defaults{$_};
    return $self->SUPER::_has_default($attr);
}

sub make_gradient {
    my($self, $gradient, $angle, $opacity) = @_;

    my @colours;
    
    for(my $i = 1; $i <= $self->{_data}->num_sets; $i++) {
        push(@colours, $self->set_clr($self->pick_data_clr($i)));
    }

    if(@colours) {
        apply_gradient($self->{graph}, \@colours, $gradient, $angle, $opacity);
    }
}


sub gradient_mixed {
    my $self = shift;

    my($width, $height) = (
        ($self->{right} - $self->{left}),
        ($self->{bottom} - $self->{top})
    );

    my @gradient =
        multi_array_gradient($self->{gradient_steps}, @{$self->{gradient}});

    $self->make_gradient(
            \@gradient, $self->{gradient_angle},
            $self->{gradient_opacity}
    );

}

sub draw_data {
    my $self = shift;
    my $rv = $self->SUPER::draw_data(@_);
    $self->gradient_mixed();
    return $rv;
}


sub _setup_boundaries {
    my $self = shift;
    $self->{left} = 0;
    $self->{top} = 0;
    $self->{right} = $self->{width};
    $self->{bottom} = $self->{height};
    $self->{x_offset} = 0;
    return $self;
}

sub draw_axes {}

sub draw_x_ticks { return shift; }
    
sub draw_y_ticks { return shift; }
