#!perl

package Sophos::Reports::Graphic::GradientBars;

use strict;
use warnings;
use base q(GD::Graph::bars);

use Math::Gradient qw(multi_array_gradient);
use GD;
use Sophos::GD::Gradient qw(apply_gradient);

our %Defaults = (
    gradient            =>      [ [ 255, 255, 255, 255 ], [ 255, 255, 255, 0 ] ],
    gradient_angle      =>      0,
    gradient_steps      =>      200,
    gradient_opacity    =>      0.3,
    cumulate            =>      1,
    bar_spacing         =>      2,
    y_long_ticks        =>      1,
);

return 1;

sub merge_hotspots {
    my $self = shift;
    my %merged;
    for (my $i = 1; $i <= $self->{_data}->num_sets; $i++) {
        foreach my $hotspot (@{$self->{_hotspots}->[$i]}) {
	    next unless ref $hotspot eq 'ARRAY';
            my($type, $x1, $y1, $x2, $y2) = @$hotspot;
            next unless $type eq 'rect';

            my($yy1, $xx2, $yy2);

            if($merged{$x1}) {
                (undef, $yy1, $xx2, $yy2) = @{$merged{$x1}};
                $yy1 = $y1 if($y1 < $yy1);
                $yy2 = $y2 if($y2 > $yy2);
                $xx2 = $x2 if($x2 > $xx2);
            } else {
                ($yy1, $xx2, $yy2) = ($y1, $x2, $y2);
            }
            $merged{$x1} = [ $x1, $yy1, $xx2, $yy2 ];
        }
    }
    return values(%merged);
}

sub gradient_bars {
    my $self = shift;
    my @gradient =
        multi_array_gradient($self->{gradient_steps}, @{$self->{gradient}});
        
    my @hotspots = $self->merge_hotspots;

    foreach my $hotspot (@hotspots) {
        my($x1, $y1, $x2, $y2) = @$hotspot;
            
        my($width, $height) = (
            int($x2 - $x1),
            int($y2 - $y1),
        );
            
        my $image = GD::Image->new($width, $height);
            
        $image->copy(
            $self->{graph}, 0, 0, $x1, $y1, $width, $height
        );

            
        apply_gradient(
            $image, [], \@gradient, $self->{gradient_angle},
            $self->{gradient_opacity}
        );
        
        $self->{graph}->copy(
            $image, $x1, $y1, 0, 0, $width, $height
        );
    }
}

sub initialise {
    my $self = shift;
    my $rv = $self->SUPER::initialise(@_);
    while(my($k, $v) = each(%Defaults)) {
        $self->set($k,$v);
    }
    $rv;
}

sub _has_default {
    my($self, $attr) = @_;
    return 1 if exists $Defaults{$_};
    return $self->SUPER::_has_default($attr);
}

sub draw_data {
    my $self = shift;
    my $rv = $self->SUPER::draw_data(@_);
    $self->gradient_bars();
    return $rv;
}
