#!perl

package Sophos::Reports::Parameters;

use strict;
use warnings (FATAL => 'all');
use Exporter;
use base q(Exporter);

sub parameters_to_hash ($);

our @EXPORT = qw(parameters_to_hash);

return 1;

sub parameters_to_hash ($) {
    (my $url = shift) =~ s#^/?##;
    my ($parameters,$output) = $url =~ m#^([^.?]+)(?:\.([^/]*))?#
	or die "Invalid parameters";

    $output ||= 'html';
    die "No parameters" unless $parameters;

    my @split = split(m{/}, $parameters);

    my $report = shift @split || die "No report parameter";
    my $type   = shift @split || die "No type parameter";
    my $date   = shift @split || die "No date parameter";
    my $span   = shift @split || die "No span parameter";
    die "Too many parameters" if @split;

    my $small = $type =~ s/^small//i;

    $report =~ s/__/::/g;

    return (
        report   => $report,
        type     => $type,
	date     => $date,
        span     => $span,
        output   => $output,
	small    => $small,
    );
}

=pod

=head1 NAME

Sophos::Reports::Parameters - Process reports parameters

=head1 DESCRIPTION

This package provides one function for translating a string representation
of reports parameters into something useful to the reports engine:

=over

=item parameters_to_hash

Accepts a string in the format:

  report_name/report_type/[end-]interval[.display_type]

Returns a set of key/value pairs representing the report to be generated.

=over

=item Parameter String

=over

=item report_name

The name of the report

=item report_type

The report's type - bars, lines, or table. Defaults to 'lines'.

=item end

Datestamp of the end of the report. The default is usually today at 23:59:59.
If specified, the datestamp must be in the format "YYYYMMDDHHMMSS", or
"YYYYMMDD", in which case 23:59:59 will be assumed.

=item interval

An SQL-friendly interval with the spaces removed, eg "1day", "2week", etc.
Defaults to "1day".

=item display_type

What format to return the result in. Defaults to "html". Valid options are:

=over

=item csv

Text file containing comma-separated values (CSV).

=item html

An HTML template to render a webpage for this report.

=item table

An HTML table containing the values from the report.

=item png

A PNG image file containing the values from the report rendered onto a
graph.

=back

=back

=back

=back

=cut
