package Sophos::Reports::Util;

use strict;
use warnings;
use base qw(Exporter);
use Time::Local;
use Sophos::Reports::TimePeriods qw(
    get_day get_week get_adjustment
    get_hour get_six_months get_month
);
use POSIX qw(localeconv);
use Carp qw(cluck);

our @EXPORT_OK = qw(
    number_commas align_to_duration
    round_time_hour round_time_day round_time_month
    duration_tick_boundary
);

sub number_commas {
    my $num = shift;
    my $comma = localeconv->{mon_thousands_sep} || ',';

    $num = reverse($num);
    $num =~ s{(\d\d\d)(?=\d)}{$1$comma}g;
    $num = reverse($num);
    
    return $num;
}

sub strip_remainder {
    my($number, $mod) = @_;
    return $number - ($number % $mod);
}

sub strip_remainder_and_adjust {
    my($number, $mod, $adjust) = @_;
    my $adjusted = $number + $adjust;
    return strip_remainder($adjusted, $mod) - $adjust;
}

sub align_to_duration {
    my($time, $dur) = @_;
    my $adjust = 0;
    $time ||= 0;
    $time = $time->num if ref($time);

    if($dur >= (get_day - get_adjustment)) {
        $adjust = tz_local_offset() || 0;
    }
    
    if($dur >= (get_week - get_adjustment)) {
        $adjust -= 259200;
    }

    $time = strip_remainder_and_adjust($time, $dur, $adjust);
    $time = PureMessage::Reports::Time->new($time);
    return $time;
}

sub round_time_hour {
    my $time = shift;
    if (! $time) {
	$time = time();
    }
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($time);
    my $round_time = timelocal(0, 0, $hour, $mday, $mon, $year);
    return $round_time;
}

sub round_time_day {
    my $time = shift;
    if (! $time) {
	$time = time();
    }
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($time);
    my $round_time = timelocal(0, 0, 0, $mday, $mon, $year);
    return $round_time;
}

sub round_time_month {
    my $time = shift;
    if (! $time) {
	$time = time();
    }
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($time);
    my $round_time = timelocal(0, 0, 0, 1, $mon, $year);
    return $round_time;
}

sub duration_tick_boundary {
    my $dur = shift;
    my $boundary;
    my $adjustment = get_adjustment;
    if($dur >= (PureMessage::Reports::TimePeriods->get_six_months - $adjustment)) {
	$boundary = get_month;
    } elsif($dur >= (PureMessage::Reports::TimePeriods->get_month - $adjustment)) {
        # 4 weeks or more - draw 1 tick a week
        $boundary = get_week;
    } elsif($dur >= (PureMessage::Reports::TimePeriods->get_week - $adjustment)) {
        # 1 week or more - draw 1 tick a day
	$boundary = get_day;
    } elsif($dur > (PureMessage::Reports::TimePeriods->get_day)) {
        # more than a day, less than a week - 1 tick every 6 hrs
        $boundary = PureMessage::Reports::TimePeriods->get_hour*6;
    } elsif($dur >= (PureMessage::Reports::TimePeriods->get_day - $adjustment)){
	$boundary = PureMessage::Reports::TimePeriods->get_hour*4;
    } else {
        # less than 1 day - draw 1 tick a hour
        $boundary = get_hour;
    }
    return $boundary;
}


1;
